<?php
/*!\file login.php
 * \brief A complete loginsystem for non-secure connections
 * 
 * \todo Flat-File option
 * \author F.Satzger
 * \date 02.01.2011
 * \ingroup SecureLogin
 */

/*! \defgroup SecureLogin
 * This package provides the ability to handle a secure login procedure using an insecure connection.
 * Therefore, various techniques are used:
 * - Server only sees salted SHA-256 hashes of the user passwords
 * - ATM, salt is the username, long term plan is to replace this with a random salt.
 * - Sent data is encrypted via RSA1024
 * 
 * To achieve all those features, your users need to have a JS-capable browser and you need to have a 
 * PHP-enabled server and a MySQL-Database. 
 */

require_once 'config.php';
require_once 'RSA.php';

class SecureLogin{
/*! \brief Outputs the necessary login form
 * 
 * \param createOnlyInvisible (optional) Defaults to false. If this parameter is false, a complete login-form get generated. Else, you have to provide your own login form, which calls javascript:encrypt() to submit
 * \param signup (optional) Defaults to false. If true, a signup-form is generated instead of a login form
 */
public static function createLoginForm($createOnlyInvisible=false, $signup=false){
    $rsa=new Crypt_RSA();
    $rsa->setEncryptionMode(CRYPT_RSA_ENCRYPTION_PKCS1);
    $key=$rsa->createKey();
    $rsa->loadKey($key['publickey']);
    $conn=mysql_connect(SecureLogin_config::$db_host, SecureLogin_config::$db_user, SecureLogin_config::$db_pass,true);
    mysql_select_db(SecureLogin_config::$db_name,$conn);
    $query="insert into ".SecureLogin_config::$db_temptable_name." values (0,'".$key['privatekey']."','".time()."');";
    mysql_query($query,$conn);
    $id=mysql_insert_id($conn);
    mysql_close($conn);
    echo '<script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'jsbn.js"></script>
<script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'prng4.js"></script>
<script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'rng.js"></script>
<script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'rsa.js"></script>
<script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'base64.js"></script>
    <script language="JavaScript" type="text/javascript" src="'.SecureLogin_config::$path_js.'sha.js"></script>
'."<script type='text/javascript'>
function encrypt(){
    var rsa=new RSAKey();
	var loginname=document.getElementById(\"".SecureLogin_config::$field_login."\").value;
    var sha=new jsSHA(document.getElementById(\"".SecureLogin_config::$field_pw."\").value+loginname+\"SecureLogin\",'ASCII');
    rsa.setPublic(\"".$rsa->modulus->toHex()."\",\"".$rsa->exponent->toHex()."\");
    document.getElementById(\"".SecureLogin_config::$form_form."_pw\").value=rsa.encrypt(sha.getHash('SHA-256','HEX'));
    document.getElementById(\"".SecureLogin_config::$form_form."_login\").value=loginname;
    document.getElementById(\"".SecureLogin_config::$form_form."\").submit();
}
</script>
";
    if(!$createOnlyInvisible){
        echo '<table border="0">
    <tr><td>'.SecureLogin_config::$string_login.'</td><td><input type="text" id="'.SecureLogin_config::$field_login.'" /></td></tr>
    <tr><td>'.SecureLogin_config::$string_pw.'</td><td><input type="password" id="'.SecureLogin_config::$field_pw.'" /></td></tr>
    <tr><td colspan="2"><input type="Button" onClick="javascript:encrypt()" value="'.SecureLogin_config::$string_submit.'" /></td></tr></table>';}

    echo '<form id="'.SecureLogin_config::$form_form.'" action="" method="post">
    <input type="hidden" id="'.SecureLogin_config::$form_form.'_id" name="'.SecureLogin_config::$form_form.'_id" value="'.$id.'" />
    <input type="hidden" id="'.SecureLogin_config::$form_form.'_login" name="'.SecureLogin_config::$form_form.'_login" value="login" />
    <input type="hidden" id="'.SecureLogin_config::$form_form.'_pw" name="'.SecureLogin_config::$form_form.'_pw"/>';
    if($signup) echo '<input type="hidden" name="'.SecureLogin_config::$form_form.'_signup" />';
    echo '</form>';
        
}
   
/*! \brief Processes login/signup
 * If a login or signup using a login form created by createLoginForm($createOnlyInvisible,$signup) occured, the 
 * password gets decrypted. If a login was done, testLogin returns $error_success if login credentials are right. If 
 * signup should be done, testLogin always returns $error_success if decryption was successfull. The users are not 
 * inserted into database automatically!
 * 
 * \param[out] login contains login name if signup/login was successful
 * \param[out] passwordhash contains password hash if signup/login was successful
 * \param[out] signup true if signup occured
 */
public static function testLogin(&$login,&$passwordhash,&$signup){
    $login=0;
    $passwordhash=0;
    $signup=isset($_POST[SecureLogin_config::$form_form."_signup"]);
    if(!isset($_POST[SecureLogin_config::$form_form."_login"])){
	return SecureLogin_config::$error_no_data_provided;
    }
    $ttn=SecureLogin_config::$db_temptable_name;
    $conn=mysql_connect(SecureLogin_config::$db_host, SecureLogin_config::$db_user, SecureLogin_config::$db_pass,true);
    if(!$conn) return SecureLogin_config::$error_mysql_connection;
    mysql_select_db(SecureLogin_config::$db_name,$conn);
    //cleanup:
    if(SecureLogin_config::$max_age){
        $query="delete from ".$ttn." where timestmp<".(time()-SecureLogin_config::$max_age);
        mysql_query($query,$conn);
    }
    $query="select id,loginkey from ".$ttn." where id=".$_POST[SecureLogin_config::$form_form."_id"].";";
    $result=mysql_query($query,$conn);
    if(mysql_num_rows($result)!=1) return SecureLogin_config::$error_temporary_key_too_old;
    $temp=mysql_fetch_object($result);
    $rsa=new Crypt_RSA();
    $rsa->setEncryptionMode(CRYPT_RSA_ENCRYPTION_PKCS1);
    $rsa->loadKey($temp->loginkey);
    $query="delete from ".$ttn." where id=".$_POST[SecureLogin_config::$form_form."_id"].";";
    mysql_query($query);
    $cipher=new Math_BigInteger($_POST[SecureLogin_config::$form_form."_pw"],16);
    $passwordhash=$rsa->decrypt($cipher->toBytes());
    $login=$_POST[SecureLogin_config::$form_form."_login"];
    if(!isset($_POST[SecureLogin_config::$form_form."_signup"])){ //login
	$utn=SecureLogin_config::$db_usertable_name;
	$query="select ".$utn.".".SecureLogin_config::$db_usertable_login.",".$utn.".".SecureLogin_config::$db_usertable_pw." from ".$utn." where ".$utn.".".SecureLogin_config::$db_usertable_login."='".$login."' and ".$utn.".".SecureLogin_config::$db_usertable_pw."='".$passwordhash."';";
        $result=mysql_query($query,$conn);
        $num=mysql_num_rows($result);
	if(!$result || !$num) return SecureLogin_config::$error_login_wrong;
	if($num>1)return SecureLogin_config::$error_mysql_consistency;
    }
    mysql_close($conn);
    return SecureLogin_config::$error_success;
}
}

?>
